/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

/****************************************************************************
** $Id: application.h,v 1.42 2004/09/02 10:24:12 flitney Exp $
**
** Copyright (C) 2002 University of Oxford.  All rights reserved.
**
*****************************************************************************/

#if !defined (APPLICATION_H)
#define APPLICATION_H

#if defined(WIN32)
#pragma warning(disable:4786)
#endif

#include <qmainwindow.h>
#include "cursor.h"
#include <qlist.h>
#include "imagegroup.h" 
#include "overlaylist.h"

class QLabel;
class QString;
class QStatusBar;
class QScrollView;
class QMultiLineEdit;
class QToolBar;
class QPopupMenu;
class QWorkspace;
class FileError;
class ViewWidget;

class ApplicationWindow: public QMainWindow, 
			 public CursorObserver
{
  Q_OBJECT
public:
  ApplicationWindow();
  virtual ~ApplicationWindow();
  virtual void update(const Cursor::Handle& c);

private slots:

  void load();
  void loadFeat();
  void closeWindow();
  void about();
  void aboutQt();
  void fileMenuAboutToShow();  
  void viewMenuAboutToShow();
  void windowsMenuAboutToShow();
  void windowsMenuActivated( int id );
  void displayMessage(const QString &, int);

public slots:
  void saveOverlay();
  void addOverlay();  
  void remOverlay();
  void createMask();
  void addLookUpTable();
  void childWindowClose(QCloseEvent*);
  void orthoView();
  void lightboxView();
  void singleView();
  void openTimeSeries();  
  void openHistogram();
 
signals:
  void message(const QString&, int );

private:
  QWorkspace* m_ws;
  QMenuBar* m_mb;
  QPopupMenu* m_fileMenu;
  QPopupMenu* m_windowsMenu;
  QPopupMenu* m_toolsMenu;
  
  int m_fileOpenID,m_fileCloseID,m_fileAddOverlayID,m_fileRemOverlayID,
    m_fileCreateMaskID, m_fileSaveOverlayID,m_fileOpenFeatID;
  int m_imageAddLutID;
  int m_viewOrthoID,m_viewLightID,m_viewSingleID,
      m_viewHistID,m_viewTimeID,m_viewTimeGridID;
  void loadFile(const QString & absFilePath);
  void buildMenus();
  bool checkAbsFilePath(const QString & absFilePath, QString ext);
  void setupStatusBar();
  void constructToolBar();
  void connectControls();
  bool windowListEmpty();
  int  windowListCount();
  void catchFileError(FileError);
  void viewShow(ViewWidget* v);
  bool tarnishCheck(Image::Handle &);
  bool tarnishCheck();
  void setCurrentDir(QString);
  void removeExtensions(QString & fileName);
  bool checkFilesExist(const QString & baseName, bool justImg); 
  bool checkSpecificFilesExist(const QString & baseName, bool useGz);
  bool checkForDuplicates(const QString & fn);
  OverlayList::Handle activeOverlayList();
  OverlayList::Handle copyActiveOverlayList();

  QString            m_filename;
  QStatusBar        *m_statusBar;

  QPopupMenu        *m_viewMenu;
  QScrollView       *m_sv;

  ImageGroup::Handle m_imageGroup;
  Cursor::Handle     m_cursor;

protected:
  virtual void closeEvent(QCloseEvent*);
};


#endif
