/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include <algorithm>
#include "cursor.h"

Cursor::Cursor(CursorObserver *o,
               short xMax, short yMax, short zMax, short vMax): 
               m_xMax(xMax), m_yMax(yMax), m_zMax(zMax),m_vMax(vMax),
               m_repaint(false)
{
  setXYZ(0,0,0);
  setV(0);
  attach(o);
}

Cursor::Cursor(short x, short y, short z, short v,
               short xMax, short yMax, short zMax, short vMax):
               m_xMax(xMax),m_yMax(yMax),m_zMax(zMax),
               m_repaint(false)
{
  setXYZ(x,y,z);
  setV(v);
}

Cursor::Handle Cursor::create(CursorObserver *o, 
                              short maxX, short maxY, short maxZ, short maxV) 
{
  Handle c = Handle(new Cursor(o,maxX,maxY,maxZ,maxV));
  c->setCountedThis(c);
      
  return c;
}

void Cursor::setCursor(short x, short y, short z)
{
  setXYZ(x,y,z);
  notify();
}

void Cursor::setCursorRepaint(short x, short y, short z)
{
  setXYZ(x,y,z);
  m_repaint = true;
  notify();
  m_repaint = false;
}

void Cursor::repaint()
{
  m_repaint = true;
  notify();
  m_repaint = false;
}

Cursor::Handle Cursor::clone()
{
  return Handle(new Cursor(m_x, m_y, m_z, m_v,
                           m_xMax, m_yMax, m_zMax, m_vMax));
}

void Cursor::setCursor(short x, short y, short z, short v)
{
  setV(v);
  setCursor(x, y, z);
}

void Cursor::attach(CursorObserver *o)
{
  m_observers.push_back(o);
}

void Cursor::detach(CursorObserver *o)
{
  m_observers.remove(o);
}

struct Update
{
  Update(Cursor::Handle c): m_cursor(c) {}

  void operator()(CursorObserver *v)
  {
    v->update(m_cursor);
  }

  const Cursor::Handle m_cursor;
};

void Cursor::notify() const
{
  std::for_each(m_observers.begin(), m_observers.end(), 
                Update(countedThis()));
}

void Cursor::setXYZ(short x,short y,short z)
{
  m_x = std::max(0, (int)x); m_x = std::min((int)m_xMax - 1, (int)m_x);
  m_y = std::max(0, (int)y); m_y = std::min((int)m_yMax - 1, (int)m_y);
  m_z = std::max(0, (int)z); m_z = std::min((int)m_zMax - 1, (int)m_z);
}

void Cursor::setV(short v)
{
  m_v = std::max(0, (int)v); m_v = std::min((int)m_vMax - 1, (int)m_v);
}
