/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if !defined(CURSORWIDGET_H)
#define CURSORWIDGET_H

#include "cursor.h"
#include <qwidget.h>
#include "overlaylist.h"
#include "tracker.h"

class QSpinBox;
class QLineEdit;
class QDoubleValidator;

class CursorWidget : public QWidget, CursorObserver, OverlayListObserver
{
  Q_OBJECT
public:
  CursorWidget(QWidget *parent,const Cursor::Handle& c,OverlayList::Handle ol);
  virtual ~CursorWidget();

  virtual void update(const Cursor::Handle& c);
  virtual void update(const OverlayList* ol, OverlayListMsg msg);

private:
  Cursor::Handle m_cursor;
  OverlayList::Handle   m_overlayList;
  QSpinBox*  m_xVoxBox;
  QSpinBox*  m_yVoxBox;
  QSpinBox*  m_zVoxBox;
  QLineEdit* m_xMmBox;
  QLineEdit* m_yMmBox;
  QLineEdit* m_zMmBox;
  QLineEdit* m_valBox;
  QDoubleValidator* m_xBoxValidator;
  QDoubleValidator* m_yBoxValidator;
  QDoubleValidator* m_zBoxValidator;

  void blockBoxSignals(bool);
  void updateValBox();
  void setInputValidators();
  QString fixMmBoxVal(QDoubleValidator*,QString &);
  int m_xVox,m_yVox,m_zVox;
  int m_xMm, m_yMm, m_zMm;

private slots:
  void voxBoxChanged(int);
  void mmBoxChanged(const QString & );

};

#endif
