/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if !defined(DRAWWIDGET_H)
#define DRAWWIDGET_H

#if defined(WIN32)
#pragma warning (disable:4786)
#endif

#include <qwidget.h>
#include <list>

class QSpinBox;
class DrawWidget;
class QToolButton;

class DrawWidgetObserver
{
public:

  virtual ~DrawWidgetObserver() {}
  virtual void update(const DrawWidget* w) = 0;

  DrawWidgetObserver() {}
};

class DrawWidget : public QWidget
{
  Q_OBJECT
public:
  typedef enum {Pen,Erase,Fill} Mode;

  DrawWidget(QWidget *parent);
  virtual ~DrawWidget();
  void attach(DrawWidgetObserver* o);
  void detach(DrawWidgetObserver* o);
  void notify();
  int   inqPixelSize();
  int   inqPixelValue();
  inline bool  inqUndoPressed() const;
  inline int   inqMode() const;
  void  setMode(int);
  void  setPrevMode();
private:
  
  QSpinBox*    m_valBox; 
  QSpinBox*    m_sizeBox;
  QToolButton* m_undoButton;
  QToolButton* m_penButton;
  QToolButton* m_eraseButton;
  QToolButton* m_fillButton;
  int          m_mode;
  int          m_penSize;
  int          m_penValue;
  int          m_eraseSize;
  int          m_eraseValue;
  int          m_fillSize;
  int          m_fillValue;
  bool         m_undoPressed;
  int          m_prevMode;

  std::list<DrawWidgetObserver*> m_observers;

  void updateControls();
  void blockControlSignals(bool);

private slots:
  void sizeBoxChanged(int);
  void valBoxChanged(int);
  void undoButtonPressed();  
  void fillButtonToggled(bool);
  void penButtonToggled(bool);
  void eraseButtonToggled(bool);

};


inline bool  DrawWidget::inqUndoPressed() const {return m_undoPressed;}
inline int   DrawWidget::inqMode() const {return m_mode;}
#endif
