/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if !defined(MDIWINDOW_H)
#define MDIWINDOW_H

#include <qmainwindow.h>

#include "imagegroup.h"
#include "cursor.h"

class ViewWidget;
class QStatusBar;
class QScrollView;
class QSpinBox;
class QLabel;
class QSplitter;
class QWorkspace;

class MDIWindow: public QMainWindow, public CursorObserver
{
  Q_OBJECT
public:
  MDIWindow( QWidget* parent, const char* name, int wflags );
  ~MDIWindow();

  void load( const QString& fn );
  void save();
  void saveAs();

  virtual void update(const Cursor* c);

signals:
  void message(const QString&, int );
  void volumeChanged(int n);
  void zoomChanged(int n);
  void modeChanged(int m);

public slots:
  void setVolume(int);
  void setZoom(int);
  void setCursorMode();
  void setBriconMode();
  void setZoomMode();
  void addOverlay();
  void orthoView();
  void openTimeSeries();
  void lightboxView();
  void detailDialog();
  void showAnalysisArea();
private:
  void buildMenus();
  void setupStatusBar();
  void constructToolBar();
  void connectControls();
  void setView(ViewWidget* newView);
  void openAnalysisArea();
  void closeAnalysisArea();
  QLabel            *m_cursorLabel;
  QString            m_filename;
  QStatusBar        *m_statusBar;
  QSpinBox          *m_volSpinBox;
  QSpinBox          *m_zoomSpinBox;
  ViewWidget        *m_view;
  ViewWidget        *m_graphView;
  QPopupMenu        *m_viewMenu;
  QScrollView       *m_sv;
  QScrollView       *m_svB;
  QWorkspace        *m_ws;
  QSplitter         *m_splitter;
  ImageGroup::Handle m_imageGroup;
  Cursor            *m_cursor;
  int                m_menuIDAnalysis;
};

#endif
