/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include "orthowidget.h"


#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qwidget.h>
#include <qtimer.h>
#include <qpixmap.h>
#include <qspinbox.h>
#include <qstatusbar.h>
#include <qpushbutton.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include "overlaywidget.h"
#include "tracker.h"
#include "ortho.xpm"


OrthoWidget::OrthoWidget(QWidget *parent,        ImageGroup::Handle& i,
                         OverlayList::Handle ol,Cursor::Handle& c): 
  ImageWidget(parent, i,ol,c), m_image(i)
{
  TRACKER("OrthoWidget::OrthoWidget(QWidget *, Cursor*, ImageGroup::Handle&)");

  m_centralWidget = new QWidget(this); 

  m_grid = new QGridLayout(m_centralWidget, 2, 2, 6, -1);
  m_centralWidget->setBackgroundColor(QColor(128, 128, 128));

  setIcon( QPixmap(ortho_xpm) );
 
  m_slices = SliceListHandle(new SliceList);
 

  SliceWidget::Handle coronal = 
    SliceWidget::Handle(new CoronalWidget(m_centralWidget, "coronal", m_cursor,
					  m_overlayList, m_briconWidget, m_drawWidget, m_undoList));

  SliceWidget::Handle axial = 
    SliceWidget::Handle(new AxialWidget(m_centralWidget, "axial", m_cursor, 
					m_overlayList, m_briconWidget, m_drawWidget, m_undoList));

  SliceWidget::Handle sagittal = 
    SliceWidget::Handle(new SagittalWidget(m_centralWidget, "sagittal", m_cursor, 
					   m_overlayList, m_briconWidget,m_drawWidget,m_undoList));
 

  ImageInfo::Handle info(m_image->getMainImage()->getInfo());


  m_grid->addWidget(coronal.get(),  0, 0);
  m_grid->addWidget(sagittal.get(), 0, 1);
  m_grid->addWidget(axial.get(),    1, 0);

  m_slices->push_back(coronal);
  m_slices->push_back(sagittal);
  m_slices->push_back(axial);

  connect(this, SIGNAL(modeChanged(int)), coronal.get(), SLOT(setMode(int)));
  connect(this, SIGNAL(modeChanged(int)), sagittal.get(), SLOT(setMode(int)));
  connect(this, SIGNAL(modeChanged(int)), axial.get(), SLOT(setMode(int)));

  connect(this, SIGNAL(zoomChanged(int)), coronal.get(), SLOT(setZoom(int)));
  connect(this, SIGNAL(zoomChanged(int)), sagittal.get(), SLOT(setZoom(int)));
  connect(this, SIGNAL(zoomChanged(int)), axial.get(), SLOT(setZoom(int)));
 
  connect(this, SIGNAL(resetZoom()), coronal.get(), SLOT(resetZoom()));
  connect(this, SIGNAL(resetZoom()), sagittal.get(), SLOT(resetZoom()));
  connect(this, SIGNAL(resetZoom()), axial.get(), SLOT(resetZoom()));
  
  connect(coronal.get(),  SIGNAL(message(const QString&, int )), this ,SIGNAL(message(const QString&, int )));
  connect(sagittal.get(), SIGNAL(message(const QString&, int )), this ,SIGNAL(message(const QString&, int )));
  connect(axial.get(),    SIGNAL(message(const QString&, int )), this ,SIGNAL(message(const QString&, int )));

  connect(this, SIGNAL(crossHairModeChanged(bool)), coronal.get(), SLOT(crossHairMode(bool)));
  connect(this, SIGNAL(crossHairModeChanged(bool)), sagittal.get(), SLOT(crossHairMode(bool)));
  connect(this, SIGNAL(crossHairModeChanged(bool)), axial.get(), SLOT(crossHairMode(bool)));

  connect(coronal.get(), SIGNAL(emitZoomFactor(int)),this, SLOT(setZoomCombo(int)));
  connect(sagittal.get(),SIGNAL(emitZoomFactor(int)),this, SLOT(setZoomCombo(int)));
  connect(axial.get(),   SIGNAL(emitZoomFactor(int)),this, SLOT(setZoomCombo(int)));

  m_cursorModeButton->setOn(true);
  emit modeChanged(SliceWidget::Cursing);

  m_cursor->attach(this);

  std::for_each(m_slices->begin(), m_slices->end(), SetImageCursor(m_cursor));

  setCentralWidget(m_centralWidget);
}

OrthoWidget::~OrthoWidget()
{
  TRACKER("OrthoWidget::~OrthoWidget");  
  m_cursor->detach(this);
}



