/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if !defined (OVERLAYDIALOG_H)
#define OVERLAYDIALOG_H

#include "overlaylist.h"
#include <qdialog.h>

class QLabel;
class QComboBox;
class QToolButton;
class QPushButton;
class QGridLayout;
class QLineEdit;
class QRegExpValidator;


class OverlayDialog : public QDialog, OverlayListObserver
{

 Q_OBJECT

 public:
  OverlayDialog(QWidget* w,OverlayList::Handle l,ImageGroup::Handle i);
  virtual ~OverlayDialog();
  void update(const OverlayList* l, OverlayListMsg msg);

 private:
  void clearDialog();
  void updateDialog();
  OverlayList::Handle m_overlayList; 
  ImageGroup::Handle  m_imageGroup;
  QLineEdit*     m_overlayEdit;
  QLabel*        m_voxLabel;
  QLabel*        m_dimLabel;
  QLabel*        m_volLabel;
  QLabel*        m_bitsLabel;
  QComboBox*     m_lutCombo;
  QComboBox*     m_dtiCombo;
  QComboBox*     m_modCombo;
  QPushButton*   m_lutButton; 
  QPushButton*   m_okButton;
  QGridLayout*   m_grid;

  QRegExpValidator* m_nameValidator;
  bool m_blockOverlayListUpdate;
  

private slots:

   void lutComboChanged(int n);
   void dtiComboChanged(int n);
   void modComboChanged(int n);
   void lutButtonPressed(); 
   void okButtonPressed();
   void overlayTextChanged( const QString & );

signals: 

   virtual void openLookUpTable();
   virtual void message(const QString&, int);
};


#endif
