/*  mixture_model.cc

Mark Woolrich, FMRIB Image Analysis Group

Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    
    LICENCE
    
    FMRIB Software Library, Release 3.2beta (c) 2004, The University of
    Oxford (the "Software")
    
    The Software remains the property of the University of Oxford ("the
    University").
    
    The Software is distributed "AS IS" under this Licence solely for
    non-commercial use in the hope that it will be useful, but in order
    that the University as a charitable foundation protects its assets for
    the benefit of its educational and research purposes, the University
    makes clear that no condition is made or to be implied, nor is any
    warranty given or to be implied, as to the accuracy of the Software,
    or that it will be suitable for any particular purpose or for use
    under any specific conditions. Furthermore, the University disclaims
    all responsibility for the use which is made of the Software. It
    further disclaims any liability for the outcomes arising from using
    the Software.
    
    The Licensee agrees to indemnify the University and hold the
    University harmless from and against any and all claims, damages and
    liabilities asserted by third parties (including claims for
    negligence) which arise directly or indirectly from the use of the
    Software or the sale of any products based on the Software.
    
    No part of the Software may be reproduced, modified, transmitted or
    transferred in any form or by any means, electronic or mechanical,
    without the express permission of the University. The permission of
    the University is not required if the said reproduction, modification,
    transmission or transference is done without financial return, the
    conditions of this Licence are imposed upon the receiver of the
    product, and all original and amended source code is included in any
    transmitted product. You may be held legally responsible for any
    copyright infringement that is caused or encouraged by your failure to
    abide by these terms and conditions.
    
    You are not permitted under this Licence to use this Software
    commercially. Use for which any financial return is received shall be
    defined as commercial use, and includes (1) integration of all or part
    of the source code or the Software into a product for sale or license
    by or on behalf of Licensee to third parties or (2) use of the
    Software or any derivative of it for research with the final aim of
    developing software products for sale or license to a third party or
    (3) use of the Software or any derivative of it for research with the
    final aim of developing non-software products for sale or license to a
    third party, or (4) use of the Software to provide any service to an
    external organisation for which payment is received. If you are
    interested in using the Software commercially, please contact Isis
    Innovation Limited ("Isis"), the technology transfer company of the
    University, to negotiate a licence. Contact details are:
    innovation@isis.ox.ac.uk quoting reference DE/1112. */

#include "mixture_model.h"
#include "utils/log.h"
#include "miscmaths/miscmaths.h"
#include "miscmaths/miscprob.h"
#include "libvis/miscplot.h"
#include "libvis/miscpic.h"
#include "newmat.h"
#include "utils/tracer_plus.h"
#include "mmoptions.h"
#include "newimage/newimagefns.h"
#include "miscmaths/sparsefn.h"
#include <utility>

using namespace NEWIMAGE;
using namespace Utilities;
using namespace MISCMATHS;
using namespace NEWMAT;
using namespace MISCPLOT;
using namespace MISCPIC;

// VOXEL ANISOTROPY

namespace Mm {

  string float2str(float f,int width, int prec, bool scientif)
  {
    ostrstream os;
    int redw = int(std::abs(std::log10(std::abs(f))))+1;
    if(width>0)
      os.width(width);
    if(scientif)
      os.setf(ios::scientific);
    os.precision(redw+std::abs(prec));
    os.setf(ios::internal, ios::adjustfield);
    os << f << '\0';
    return os.str();
  } 

  // void GammaDistribution::validate() {
    
//     // want mn to be positive
//       if(mn <= minmode)
// 	{
// 	  mn = minmode;
// 	}

//       // want mode to be positive
//       float mode = mn - var/mn;

//       if(mode <= minmode)
// 	{
// //  	  OUT("Gamma::validate mode greater than minmode");
// //  	  OUT(mn);
// //  	  OUT(var);
// //  	  OUT(mode);
// //  	  OUT(minmode);
// 	  float n = std::abs(minmode);
// 	  mn = std::sqrt(Sqr(n)/4.0+var)+n/2.0;
// //  	  OUT(mn);
// //  	  OUT(mn - var/mn);
// 	}

// //        OUT(var);
// //        OUT(mode);
// //        OUT(minmode);
// //        OUT(mn);

//     }

  bool GammaDistribution::validate() {
    
    // want mn to be positive
      if(mn <= minmode)
	{
	  return false;
	}

      // want mode to be positive
      float mode = mn - var/mn;

      if(mode <= minmode)
	{

	  return false;
	}

      return true;
    }

//   void FlippedGammaDistribution::validate() {
    
// 	// want mn to be negative
// 	if(mn >= -std::abs(minmode))
// 	  {
// 	    //	  OUT(mn);
// 	    mn = -std::abs(minmode);
// 	  }
	
// 	// want mode to be !positive!
// 	float mode = abs(mn) - var/abs(mn);
	
// 	if(mode <= std::abs(minmode))
// 	  {
// 	    //  	  OUT("FlippedGamma::validate mode less than minmode");
// 	    //  	  OUT(mn);
// 	    //  	  OUT(var);
// 	    //  	  OUT(mode);
// 	    float n = std::abs(minmode);
// 	    mn = -(std::sqrt(Sqr(n)/4.0+var)+n/2.0);
	    
// 	    //  	  OUT(minmode);
// 	    //  	  OUT(mn);
// 	  }

// //        OUT(var);
// //        OUT(mode);
// //        OUT(minmode);
// //        OUT(mn);
//     }

  bool FlippedGammaDistribution::validate() {
        
  
	// want mn to be negative
	if(mn >= -std::abs(minmode))
	  {
	    //	  OUT(mn);
	    //OUT(false);
	    return false;
	  }
	
	// want mode to be !positive!
	float mode = abs(mn) - var/abs(mn);
	
	if(mode <= std::abs(minmode))
	  {
	    //  	  OUT("FlippedGamma::validate mode less than minmode");
	    //  	  OUT(mn);
	    //  	  OUT(var);
	    //  	  OUT(mode);
	    //OUT(false);
	    return false;
	    
	    //  	  OUT(minmode);
	    //  	  OUT(mn);
	    
	  }

//        OUT(var);
//        OUT(mode);
//        OUT(minmode);
//        OUT(mn);
	//OUT(true);
	return true;

    }

  SmmFunction::SmmFunction(const ColumnVector& pdata, vector<Distribution*>& pdists, const float& pmrf_precision, const volume<int>& pmask, const vector<Connected_Offset>& pconnected_offsets, const volume<int>& pindices, float plambda, float plog_bound)
      : gEvalFunction(),
	data(pdata),
	dists(pdists),
	mrf_precision(pmrf_precision),
	mask(pmask),
	connected_offsets(pconnected_offsets),
	indices(pindices),
	num_superthreshold(pdata.Nrows()),
	nclasses(pdists.size()),
	lambda(plambda),
	log_bound(plog_bound)
    {}

  float SmmFunction::evaluate(const ColumnVector& m_tildew) const
  {
    Tracer_Plus trace("SmmFunction::evaluate");

    float ret = 0.0;

    // stuff from MRF term:
    for(int z = 0; z < mask.zsize(); z++)
      for(int y = 0; y < mask.ysize(); y++)
	for(int x = 0; x < mask.xsize(); x++)
	  if(mask(x,y,z))
	    {
	      int xi=0,yi=0,zi=0;
	      for(unsigned int i = 0; i < connected_offsets.size(); i++) 
		{
		  xi = x+connected_offsets[i].x;
		  yi = y+connected_offsets[i].y;
		  zi = z+connected_offsets[i].z;
		  		  
		  if(mask(xi,yi,zi))
		    {
		      for(int c = 0; c < nclasses; c++)
			{
			  ret +=  Sqr(m_tildew(c*num_superthreshold+indices(x,y,z)) - m_tildew(c*num_superthreshold+indices(xi,yi,zi)));
			}
		    }
		}
	    }

    ret *= mrf_precision/4.0;

    // likelihood terms stuff:
    //    float A = 1.0/std::sqrt(2*M_PI);
    
    for(int r = 1; r<=num_superthreshold; r++)
      {       
	float h = 0;
       
	RowVector wtilde(nclasses);
	wtilde = 0;

	for(int c=0; c < nclasses; c++)
	  {
	    wtilde(c+1) = m_tildew(c*num_superthreshold+r);

	    if(std::abs(wtilde(c+1))>log_bound) return 1e32;
	  }

	//	OUT(wtilde);
	RowVector w = Mixture_Model::transform(wtilde,lambda,log_bound);
	//OUT(w);

	for(int c=1; c <= nclasses; c++)
	  {
	    h += w(c)*dists[c-1]->pdf(data(r));
	  }
       
	ret += -std::log(h);
      }

    return ret;
  }

  ReturnMatrix SmmFunction::g_evaluate(const ColumnVector& m_tildew) const
  {
    Tracer_Plus trace("SmmFunction::g_evaluate");
        
    //    float A = 1.0/std::sqrt(2*M_PI);        
    ColumnVector derivative_tilde(num_superthreshold*nclasses);
    derivative_tilde = 0;

    // first derivative stuff from MRF term:
    for(int z = 0; z < mask.zsize(); z++)
      for(int y = 0; y < mask.ysize(); y++)
	for(int x = 0; x < mask.xsize(); x++)
	  if(mask(x,y,z))
	    {
	      ColumnVector derivsum(nclasses);
	      derivsum = 0;

	      int xi=0,yi=0,zi=0;
	      for(unsigned int i = 0; i < connected_offsets.size(); i++) 
		{
		  xi = x+connected_offsets[i].x;
		  yi = y+connected_offsets[i].y;
		  zi = z+connected_offsets[i].z;
		  		  
		  if(mask(xi,yi,zi))
		    {
		      for(int c = 0; c < nclasses; c++)
			{  
			  derivsum(c+1) +=  m_tildew(c*num_superthreshold+indices(x,y,z)) - m_tildew(c*num_superthreshold+indices(xi,yi,zi)); 
			}
		    }
		}

	      for(int c = 0; c < nclasses; c++)
		{		  
		  derivative_tilde(c*num_superthreshold+indices(x,y,z)) += derivsum(c+1)*mrf_precision;
		}
	    }

    // likelihood terms stuff:
    for(int r = 1; r<=num_superthreshold; r++)
      {
	RowVector wtilde(nclasses);
	wtilde = 0;
	
	for(int c=0; c < nclasses; c++)
	  wtilde(c+1) = m_tildew(c*num_superthreshold+r);
		
	RowVector w = Mixture_Model::logistic_transform(wtilde,lambda,log_bound);
	
	RowVector R(nclasses);
	R = 0;
	float P = 0;
	float h = 0;
	//float Q = 0;

	for(int c=1; c <= nclasses; c++)
	  {	
	    // calculate terms for use in deriv of likelihood:
	    h += w(c)*dists[c-1]->pdf(data(r));
	    R(c) = boundexp(wtilde(c)/(lambda*log_bound));
	    P += R(c);
	  }

//  	OUT(r);
//   	OUT(wtilde);
//   	OUT(w);
//  	OUT(P);

	// calculate dw/dy and dy/dx
	vector<ColumnVector> dwdy(nclasses);
	for(int k=1; k <= nclasses; k++)
	  {	
	    dwdy[k-1].ReSize(nclasses);
	    dwdy[k-1] = 0;
//  	    dydx[k-1].ReSize(nclasses);
//  	    dydx[k-1] = 0;
	   
	    for(int c2=1; c2 <= nclasses; c2++)
	      {		
		if(c2==k)
		  {
		    dwdy[k-1](k) = R(k)*(1-R(k)/P)/(lambda*log_bound*P);
		  }
		else
		  {
		    dwdy[k-1](c2) = -R(k)*R(c2)/(lambda*log_bound*Sqr(P));
		    
		  }	       
	      }
	  }

	// calculate df/dw
	ColumnVector dfdw(nclasses);
	dfdw = 0;       

	for(int c=1; c <= nclasses; c++)
	  {
	    dfdw(c) = -dists[c-1]->pdf(data(r))/h;
	  }
           
    	// Now fill up derivative vector for tildew (aka x) for this voxel
	ColumnVector derivanal(nclasses);
	derivanal = 0;
	
	for(int k=1; k <= nclasses; k++)
	  {
	    float sum_l = 0;
	    for(int l=1; l <= nclasses; l++)
	      {
		sum_l += dfdw(l)*dwdy[l-1](k);
	      }
	    
	    derivanal(k) = sum_l;
	  }

	// now fill up return vector
	for(int k=1; k <= nclasses; k++)
	  {
	    derivative_tilde((k-1)*num_superthreshold+r) += derivanal(k);
	  }

      }

    
    derivative_tilde.Release();
    
    return derivative_tilde;
  }

  SmmFunctionDists::SmmFunctionDists(const ColumnVector& pdata, vector<Distribution*>& pdists, const float& pmrf_precision, const volume<int>& pmask, const vector<Connected_Offset>& pconnected_offsets, const volume<int>& pindices, float plambda, float plog_bound, const ColumnVector& pm_tildew)
      : gEvalFunction(),
	data(pdata),
	dists(pdists),
	mrf_precision(pmrf_precision),
	mask(pmask),
	connected_offsets(pconnected_offsets),
	indices(pindices),
	num_superthreshold(pdata.Nrows()),
	nclasses(pdists.size()),
	lambda(plambda),
	log_bound(plog_bound),
	m_tildew(pm_tildew)
    {}

  float SmmFunctionDists::evaluate(const ColumnVector& x) const
  {
    Tracer_Plus trace("SmmFunctionDists::evaluate");

    float ret = 0.0;

    // get dists params from passed in vector
    for(int c = 0; c < nclasses; c++)
      {
	if(!dists[c]->setparams(x(c*2+1),x(c*2+2),1))
	  return 1e32;
      }

    // likelihood terms stuff:
    //    float A = 1.0/std::sqrt(2*M_PI);
    
    for(int r = 1; r<=num_superthreshold; r++)
      {       
	float h = 0;
       
	RowVector wtilde(nclasses);
	wtilde = 0;

	for(int c=0; c < nclasses; c++)
	  {
	    wtilde(c+1) = m_tildew(c*num_superthreshold+r);

	    if(std::abs(wtilde(c+1))>log_bound) 
	      {
		return 1e32;
	      }
	  }

	//	OUT(wtilde);
	RowVector w = Mixture_Model::transform(wtilde,lambda,log_bound);
	//OUT(w);

	for(int c=1; c <= nclasses; c++)
	  {
	    h += w(c)*dists[c-1]->pdf(data(r));
	  }
       
	ret += -std::log(h);
      }

    return ret;
  }

  ReturnMatrix SmmFunctionDists::g_evaluate(const ColumnVector& x) const
  {
    Tracer_Plus trace("SmmFunctionDists::g_evaluate");

    // get dists params from passed in vector
    for(int c = 0; c < nclasses; c++)
      {
	dists[c]->setparams(x(c*2+1),x(c*2+2),1);	
      }      

    // likelihood terms stuff:
    ColumnVector meansum(nclasses); 
    meansum = 0.0;
    ColumnVector varsum(nclasses); 
    varsum = 0.0;
    for(int r = 1; r<=num_superthreshold; r++)
      {
	RowVector wtilde(nclasses);
	wtilde = 0;
	
	for(int c=0; c < nclasses; c++)
	  wtilde(c+1) = m_tildew(c*num_superthreshold+r);
		
	RowVector w = Mixture_Model::logistic_transform(wtilde,lambda,log_bound);
		
	float h = 0;

	//float Q = 0;

	for(int c=1; c <= nclasses; c++)
	  {	
	    // calculate terms for use in deriv of likelihood:
	    float mult=1;
	    //	    if(dists[c-1]->flipped) mult=-1;
	    h += w(c)*mult*dists[c-1]->pdf(data(r));	    
	  }	

	for(int c=1; c <= nclasses; c++)
	  {
	    meansum(c) += -w(c)*dists[c-1]->dpdfdmn(data(r))/h;
	    varsum(c) += -w(c)*dists[c-1]->dpdfdvar(data(r))/h;
	  }
      }
	
    ColumnVector derivative_tilde(nclasses*2);
    derivative_tilde = 0;
    
    for(int c = 0; c < nclasses; c++)
      {	
	derivative_tilde(c*2+1) = meansum(c+1);
	derivative_tilde(c*2+2) = varsum(c+1);
      }

    
    derivative_tilde.Release();
    
    return derivative_tilde;
  }

  Mixture_Model::Mixture_Model(const volume<float>& pspatial_data, const volume<int>& pmask, const volume<float>& pepi_example_data, float pepibt, vector<Distribution*>& pdists, vector<volume<float> >& pw_means, ColumnVector& pY, MmOptions& popts) :
    xsize(pmask.xsize()),
    ysize(pmask.ysize()),
    zsize(pmask.zsize()),
    num_superthreshold(0),
    nclasses(pdists.size()),
    spatial_data(pspatial_data),
    mask(pmask),
    epi_example_data(pepi_example_data),
    epibt(pepibt),
    localweights(),
    connected_offsets(),
    indices(pmask.xsize(),pmask.ysize(),pmask.zsize()),    
    Y(pY),
    D(),
    m_tildew(),
    covariance_tildew(),
    precision_tildew(),
    derivative_tildew(),   
    mrf_precision(popts.mrfprecstart.value()),
    nonspatial(popts.nonspatial.value()),
    niters(popts.niters.value()),    
    updatetheta(popts.updatetheta.value()),
    debuglevel(popts.debuglevel.value()),
    lambda(popts.phi.value()),
    log_bound(10),
    dists(pdists),
    w_means(pw_means),
    ntracesamps(popts.tracesamps.value()),
    mrfprecmultiplier(popts.mrfprecmultiplier.value()),
    fixmrfprec(popts.fixmrfprec.value())
  {
    connected_offsets.push_back(Connected_Offset(-1,0,0,0,1));
    connected_offsets.push_back(Connected_Offset(1,0,0,1,0));
    connected_offsets.push_back(Connected_Offset(0,-1,0,2,3));
    connected_offsets.push_back(Connected_Offset(0,1,0,3,2));
    connected_offsets.push_back(Connected_Offset(0,0,-1,4,5));
    connected_offsets.push_back(Connected_Offset(0,0,1,5,4));
  }

  Mixture_Model::Mixture_Model(const volume<float>& pspatial_data, const volume<int>& pmask, const volume<float>& pepi_example_data, float pepibt, vector<Distribution*>& pdists, vector<volume<float> >& pw_means, ColumnVector& pY, bool pnonspatial, int pniters, bool pupdatetheta, int pdebuglevel, float pphi, float pmrfprecstart, int pntracesamps, float pmrfprecmultiplier, bool pfixmrfprec) :
    xsize(pmask.xsize()),
    ysize(pmask.ysize()),
    zsize(pmask.zsize()),
    num_superthreshold(0),
    nclasses(pdists.size()),
    spatial_data(pspatial_data),
    mask(pmask),
    epi_example_data(pepi_example_data),
    epibt(pepibt),
    localweights(),
    connected_offsets(),
    indices(pmask.xsize(),pmask.ysize(),pmask.zsize()),
    Y(pY),
    D(),
    m_tildew(),
    covariance_tildew(),
    precision_tildew(),
    derivative_tildew(),    
    mrf_precision(pmrfprecstart),
    nonspatial(pnonspatial),
    niters(pniters),    
    updatetheta(pupdatetheta),
    debuglevel(pdebuglevel),
    lambda(pphi),
    log_bound(10.0),
    dists(pdists),
    w_means(pw_means),
    ntracesamps(pntracesamps),
    mrfprecmultiplier(pmrfprecmultiplier),
    fixmrfprec(pfixmrfprec)
  {
    connected_offsets.push_back(Connected_Offset(-1,0,0,0,1));
    connected_offsets.push_back(Connected_Offset(1,0,0,1,0));
    connected_offsets.push_back(Connected_Offset(0,-1,0,2,3));
    connected_offsets.push_back(Connected_Offset(0,1,0,3,2));
    connected_offsets.push_back(Connected_Offset(0,0,-1,4,5));
    connected_offsets.push_back(Connected_Offset(0,0,1,5,4));
  }

  void Mixture_Model::setup()
  {
    Tracer_Plus trace("Mixture_Model::setup");

    trace_tol = 0.0001;
    scg_tol = 0.01;

    // setup lattice:
    localweights.reinitialize(mask.xsize(),mask.ysize(),mask.zsize(),6);
    localweights = 0;
  
    for(int x = 0; x < mask.xsize(); x++)
      for(int y = 0; y < mask.ysize(); y++)
	for(int z = 0; z < mask.zsize(); z++)
	  if(mask(x,y,z))
	    {
	      num_superthreshold++;
	      
	      int xi,yi,zi;
	      for(unsigned int i = 0; i < connected_offsets.size(); i++) 
		{
		  xi = x+connected_offsets[i].x;
		  yi = y+connected_offsets[i].y;
		  zi = z+connected_offsets[i].z;
		  
		  if(mask(xi,yi,zi))
		    {
		      if(epibt > 0)
			{
			  localweights(x,y,z,connected_offsets[i].ind) = exp(-Sqr(epi_example_data(xi,yi,zi)-epi_example_data(x,y,z))/(2*epibt*epibt));			  
			}
		      else
			{
			  localweights(x,y,z,connected_offsets[i].ind) = 1;
			}
		    }
		}
	    }   
      
    OUT(nclasses);
    OUT(num_superthreshold);
    OUT(niters);
    OUT(mrf_precision);
    OUT(fixmrfprec);
    OUT(mrfprecmultiplier);
    OUT(ntracesamps);
    OUT(updatetheta);
    OUT(nonspatial);
    OUT(lambda);
    OUT(log_bound);

    w_means.resize(nclasses);    
    indices = 0;
    int index=1;
    Y.ReSize(num_superthreshold);
    Y = 0;
    D.ReSize(num_superthreshold*nclasses,num_superthreshold*nclasses);

    m_tildew.ReSize(num_superthreshold*nclasses);
    m_tildew = 0;
    covariance_tildew.ReSize(num_superthreshold*nclasses,num_superthreshold*nclasses);

    precision_tildew.ReSize(num_superthreshold*nclasses,num_superthreshold*nclasses);
 
    derivative_tildew.ReSize(num_superthreshold*nclasses);
    derivative_tildew = 0;

    ColumnVector num_neigbours(num_superthreshold);
    num_neigbours = 0;

    for(int z = 0; z < mask.zsize(); z++)    
      for(int y = 0; y < mask.ysize(); y++)	
	for(int x = 0; x < mask.xsize(); x++)
	  if(mask(x,y,z))
	    {
	      int xi=0,yi=0,zi=0;
	      for(unsigned int i = 0; i < connected_offsets.size(); i++) 
		{
		  xi = x+connected_offsets[i].x;
		  yi = y+connected_offsets[i].y;
		  zi = z+connected_offsets[i].z;
		  
		  if(mask(xi,yi,zi))
		    {
		      num_neigbours(index) += localweights(x,y,z,connected_offsets[i].ind);
		    }
		}

	      indices(x,y,z) = index;
	      Y(index) = spatial_data(x,y,z);

	      index++;
	    }

    for(int z = 0; z < mask.zsize(); z++)
      for(int y = 0; y < mask.ysize(); y++)
	for(int x = 0; x < mask.xsize(); x++)
	  if(mask(x,y,z))
	    {
	      int xi=0,yi=0,zi=0;
	      for(unsigned int i = 0; i < connected_offsets.size(); i++) 
		{
		  xi = x+connected_offsets[i].x;
		  yi = y+connected_offsets[i].y;
		  zi = z+connected_offsets[i].z;
		  
		  if(mask(xi,yi,zi))
		    {
		      for(int c = 0; c < nclasses; c++)
			{
			  D.insert(c*num_superthreshold+indices(x,y,z),c*num_superthreshold+indices(xi,yi,zi), -1.0);
			  D.insert(c*num_superthreshold+indices(xi,yi,zi),c*num_superthreshold+indices(x,y,z), -1.0);
			}
		    }
		}

	      for(int c = 0; c < nclasses; c++)
		D.insert(c*num_superthreshold+indices(x,y,z),c*num_superthreshold+indices(x,y,z),num_neigbours(indices(x,y,z)));
	    }
    
    for(int c = 1; c <= nclasses; c++)
      dists[c-1]->setuseprop(true);

    // initialise tildew
    for(int r = 1; r<=num_superthreshold; r++)
      {
	ColumnVector probs(nclasses);
	probs = 0;

	//float sum = 0;
	float tmp;
	tmp = Y(r);

	int maxc = 1;
	float maxprob = 0;
	float sumprob = 0;
	
	for(int c = 1; c <= nclasses; c++)
	  {	    
	    probs(c) = dists[c-1]->pdf(tmp);
	    sumprob += probs(c);

	    if(probs(c) > maxprob) {
	      maxc = c;
	      maxprob = probs(c);
	    }	    	    
	  }

	RowVector w(nclasses);
	w = 0;
	w(maxc) = 1;
//  	OUT(w);

	for(int c = 1; c <= nclasses; c++)
	  {
	    w(c) = probs(c)/sumprob;
	  }

	RowVector wtilde = inv_transform(w,lambda,log_bound);

	for(int c=0; c < nclasses; c++)
	  {
	    m_tildew(c*num_superthreshold+r) = wtilde(c+1);
	  }

      }

    for(int c = 1; c <= nclasses; c++)
      dists[c-1]->setuseprop(false);

    
  }

  void Mixture_Model::run()
  {
    Tracer_Plus trace("Mixture_Model::run");    

    if(!nonspatial)
      for(it=1; it<=niters; it++)
	{
	  OUT(it);
	  OUT("Updating weights");
	  update_tildew_scg();		

	  OUT("Calculating precision matrix");
	  calculate_tildew_precision();	  

	  if(!fixmrfprec)
	    {
	      OUT("Calculating trace");
	      calculate_trace_tildew_D();

	      OUT("Updating MRF precision");
	      update_mrf_precision();
	    }


	  if(updatetheta)	    
	    {
	      OUT("Updating distribution params");
	      update_theta();
	    }

	  OUT(mrf_precision);

	  cout << "Iterations=" << it << endl;
	}
        
  }    

  void Mixture_Model::update_theta()
  {
    Tracer_Plus trace("Mixture_Model::update_theta");
    
    // update means and vars using ML estimates, given weights are known
    vector<ColumnVector> weights;
    //    get_weights2(weights,100);
    get_weights(weights);

    
    if(true)
      {
	SmmFunctionDists smmfunc(Y,dists,mrf_precision,mask,connected_offsets,indices,lambda,log_bound,m_tildew);
	// set dists params in vector
	ColumnVector x(nclasses*2);
	x=0;    	
	for(int c = 0; c < nclasses; c++)
	  {	    
	    x(c*2+1) = dists[c]->getmean();
	    x(c*2+2) = dists[c]->getvar();
	  }
	
	ColumnVector tmp2(x.Nrows());tmp2=1;	
	scg(x, smmfunc, tmp2, scg_tol);
	
	// get dists params from vector
	for(int c = 0; c < nclasses; c++)
	  {
	    dists[c]->setparams(x(c*2+1),x(c*2+2),1);
	  }
      }
    else
      {
    
    for(int c = 0; c < nclasses; c++)
      {	
	float sumweights = 0;
	float sumy = 0;

	for(int r = 1; r<=num_superthreshold; r++)
	  {
	    float wtmp = weights[c](r);
	    sumweights += wtmp;
	    sumy += wtmp*Y(r);	    
	  }
	
	float mu = sumy/sumweights;
	
	float ssy = 0;
	for(int r = 1; r<=num_superthreshold; r++)
	  {
	    float wtmp = weights[c](r);	    
	    ssy += wtmp*Sqr(Y(r)-mu);
	  }

//  	OUT(c);
//  	OUT(sumweights);
//  	OUT(sumy/sumweights);
//  	OUT(ssy/sumweights);

	if(updatetheta)	
	  dists[c]->setparams(mu,ssy/sumweights,sumweights/num_superthreshold);
	else
	  dists[c]->setparams(dists[c]->getmean(),dists[c]->getvar(),sumweights/num_superthreshold);
      
      }    
      }
    meanhist.push_back(dists[0]->getmean());
  }

  void Mixture_Model::update_tildew_scg()
  {
    Tracer_Plus trace("Mixture_Model::update_tildew_scg");

    SmmFunction smmfunc(Y,dists,mrf_precision,mask,connected_offsets,indices,lambda,log_bound);
    
    //  	float tmp = smmfunc.evaluate(m_tildew);
    //  	OUT(tmp);
    ColumnVector tmp2(m_tildew.Nrows());tmp2=1;
    scg(m_tildew,smmfunc, tmp2, scg_tol);
    //  	tmp = smmfunc.evaluate(m_tildew);
    //  	OUT(tmp);
 
  }
  
  void Mixture_Model::update_mrf_precision()
  {
    Tracer_Plus trace("Mixture_Model::update_mrf_precision"); 
    
    float errorprecision = 1;
    float var = Sqr(errorprecision)*10;
    float c_0 = Sqr(errorprecision)/var;
    float b_0 = errorprecision/var;
    float c_g = nclasses*num_superthreshold/2.0 + c_0;
    float b_g = 1.0/(0.5*(quadratic(m_tildew,D) + trace_covariance_tildew_D)+1.0/b_0);

    float mrf_precision_new = boundexp(std::log(b_g)+MISCMATHS::lgam(c_g+1)-MISCMATHS::lgam(c_g));

    float mrf_precision_old = mrf_precision;      

    if(mrf_precision_hist.size()>= 2)
      {
	float mrf_precision_oldold = mrf_precision_hist[mrf_precision_hist.size()-2];     
	
// 	OUT(mrfprecmultiplier);
// 	OUT(mrf_precision_oldold);
// 	OUT(mrf_precision_old);
// 	OUT(mrf_precision_new);
// 	OUT(mrf_precision);
	
	if(sign(mrf_precision_oldold - mrf_precision_old) != sign(mrf_precision_old-mrf_precision_new))
	  {
	    mrfprecmultiplier *= 0.5;
	    OUT(mrfprecmultiplier);
	  }
      }

    mrf_precision = mrfprecmultiplier*(mrf_precision_new-mrf_precision_old)+mrf_precision_old;

    if(mrf_precision<=0)
      {
	mrf_precision = 1;
	mrfprecmultiplier *= 0.5;
	OUT(mrfprecmultiplier);
      }  
        
    mrf_precision_hist.push_back(mrf_precision);
    
  }

  void Mixture_Model::calculate_tildew_precision(bool forcepositive)
  {
    Tracer_Plus trace("Mixture_Model::calculate_tildew_precision");

    // build up precision/hessian matrix for tildew (aka x)
    //    float A = 1.0/std::sqrt(2*M_PI);
    float lamsqr = Sqr(lambda*log_bound);

    derivative_tildew.Nrows();

    derivative_tildew = 0;

    //precision_tildew.ReSize(num_superthreshold*nclasses,num_superthreshold*nclasses);
    precision_tildew = D;
        
    precision_tildew.multiplyby(mrf_precision);

    if(true)
      {
	// stuff from MRF term:
	for(int z = 0; z < mask.zsize(); z++)
	  for(int y = 0; y < mask.ysize(); y++)
	    for(int x = 0; x < mask.xsize(); x++)
	      if(mask(x,y,z))
		{
		  ColumnVector derivsum(nclasses);
		  derivsum = 0;

		  int xi=0,yi=0,zi=0;
		  for(unsigned int i = 0; i < connected_offsets.size(); i++) 
		    {
		      xi = x+connected_offsets[i].x;
		      yi = y+connected_offsets[i].y;
		      zi = z+connected_offsets[i].z;
		  		  
		      if(mask(xi,yi,zi))
			{
			  for(int c = 0; c < nclasses; c++)
			    {  
			      derivsum(c+1) +=  m_tildew(c*num_superthreshold+indices(x,y,z)) - m_tildew(c*num_superthreshold+indices(xi,yi,zi)); 
			    }
			}
		    }

		  for(int c = 0; c < nclasses; c++)
		    {		  
		      derivative_tildew(c*num_superthreshold+indices(x,y,z)) += derivsum(c+1)*mrf_precision;
		    }
	      
		}

	// likelihood terms stuff:
	for(int r = 1; r<=num_superthreshold; r++)
	  {
	    RowVector wtilde(nclasses);
	    wtilde = 0;
	    for(int c=1; c <= nclasses; c++)
	      {	
		// calculate terms for use in calculating hessian and deriv of likelihood:
		wtilde(c) = m_tildew((c-1)*num_superthreshold+r);
	      }

	    RowVector w = Mixture_Model::logistic_transform(wtilde,lambda,log_bound);

	    RowVector gamma(nclasses);
	    gamma = 0;
	    RowVector R(nclasses);
	    R = 0;
	    float P = 0;
	    float h = 0;

	    for(int c=1; c <= nclasses; c++)
	      {	
		// calculate terms for use in calculating hessian and deriv of likelihood:	    
		h += w(c)*dists[c-1]->pdf(Y(r));
		R(c) = boundexp(wtilde(c)/(lambda*log_bound));
		P += R(c);
	      }
	    //  	OUT(w);
	    //  	OUT(wtilde);
	    //  	OUT(R);
	    //  	OUT(P);
	    //  	OUT(Q);
	    //  	OUT(h);

	    // calculate dw_k/dx and d^2w_k/dx^2 
	    vector<SymmetricMatrix> dwdydy(nclasses);
	    vector<ColumnVector> dwdy(nclasses);
	    for(int k=1; k <= nclasses; k++)
	      {	
		dwdydy[k-1].ReSize(nclasses);
		dwdydy[k-1] = 0;
		dwdy[k-1].ReSize(nclasses);
		dwdy[k-1] = 0;
	    
		for(int c2=1; c2 <= nclasses; c2++)
		  {		
		
		    if(c2==k)
		      {
			dwdydy[k-1](k,k) = R(k)/(lamsqr*P)*(1-3*R(k)/P+2*Sqr(R(k)/P));
			dwdy[k-1](k) = R(k)*(1-R(k)/P)/(lambda*log_bound*P);
		      }
		    else
		      {
			dwdydy[k-1](c2,c2) = R(k)*R(c2)/(lamsqr*Sqr(P))*(2*R(c2)/P-1);	
			dwdy[k-1](c2) = -R(k)*R(c2)/(lambda*log_bound*Sqr(P));
		      }

		    for(int c3=c2+1; c3 <= nclasses; c3++)
		      {
			if(c2==k) // we know that c3 != c2
			  {
			    dwdydy[k-1](k,c3) = R(k)*R(c3)/(lamsqr*Sqr(P))*(2*R(k)/P-1);
			  }
			else if(c3==k) // we know that c3 != c2
			  {
			    dwdydy[k-1](k,c2) = R(k)*R(c2)/(lamsqr*Sqr(P))*(2*R(k)/P-1);
			  }
			else
			  {
			    dwdydy[k-1](c2,c3) = 2*R(k)*R(c2)*R(c3)/(lamsqr*Sqr(P)*P);
			  }
		      }
		  }
	      }

	    // calculate d^2f/dw^2 and df/dw
	    SymmetricMatrix dfdwdw(nclasses);
	    dfdwdw = 0;
	    ColumnVector dfdw(nclasses);
	    dfdw = 0;

	    //	float premult = Sqr(A/h);

	    for(int c=1; c <= nclasses; c++)
	      {
		dfdwdw(c,c) =  Sqr(dists[c-1]->pdf(Y(r))/h);
		dfdw(c) = -dists[c-1]->pdf(Y(r))/h;

		for(int c2=c+1; c2 <= nclasses; c2++)
		  {
		    dfdwdw(c,c2) = dists[c-1]->pdf(Y(r))*dists[c2-1]->pdf(Y(r))/Sqr(h);
		  }
	      }

	    if(debuglevel==2)
	      {
		OUT(dfdwdw);
		OUT(dfdw);
	      }

	    // Now fill up precision/Hessian for tildew (aka x) for this voxel
	    SymmetricMatrix hessanal(nclasses);
	    hessanal = 0;
	    ColumnVector derivanal(nclasses);
	    derivanal = 0;

	    for(int k=1; k <= nclasses; k++)
	      {
		// diagonal terms k=j
		float sum_l = 0;
		float sum_l2 = 0;
		for(int l=1; l <= nclasses; l++)
		  {
		    float sum_m = 0;
		    for(int m=1; m <= nclasses; m++)
		      {
			sum_m += dfdwdw(m,l)*dwdy[m-1](k);
		      }
		    sum_l += sum_m*dwdy[l-1](k) + dfdw(l)*dwdydy[l-1](k,k);

		    sum_l2 += dfdw(l)*dwdy[l-1](k);
		  }

		derivanal(k) = sum_l2;
		hessanal(k,k) = sum_l;

		// off-diagonal terms jk
		for(int n=k+1; n <= nclasses; n++)
		  {
		    float sum_l = 0;
		    for(int l=1; l <= nclasses; l++)
		      {
			float sum_m = 0;
			for(int m=1; m <= nclasses; m++)
			  {
			    sum_m += dfdwdw(m,l)*dwdy[l-1](n);
			  }
			sum_l += sum_m*dwdy[l-1](n) + dfdw(l)*dwdydy[l-1](n,k);		 
		      }

		    hessanal(n,k) = sum_l;
		  }
	      }

	    for(int k=1; k <= nclasses; k++)
	      {
		derivative_tildew((k-1)*num_superthreshold+r) += derivanal(k);
	    
		if(forcepositive)
		  precision_tildew.addto((k-1)*num_superthreshold+r,(k-1)*num_superthreshold+r,Max(0,hessanal(k,k)));	
		else
		  precision_tildew.addto((k-1)*num_superthreshold+r,(k-1)*num_superthreshold+r,hessanal(k,k));	    
	   
	      }
	
	  }
      }

    
  }

  void Mixture_Model::calculate_trace_tildew_D()
  {
    // Now calculate trace    
    float trace_new = 0; 

    if(ntracesamps > 0)
      {
	trace_new = solvefortracex(precision_tildew, D, covariance_tildew, ntracesamps, trace_tol);
	//	OUT(trace_new);
      }
    else
      {
	// for(int r=1; r <= num_superthreshold*nclasses; r++)
// 	  {
// 	    trace_new += 1.0/precision_tildew(r,r)*D(r,r);		
// 	  }

	for(int r=1; r <= num_superthreshold; r++)
	  {
	      SymmetricMatrix tmp(nclasses);
	      tmp = 0;
	      DiagonalMatrix tmp2(nclasses);
	      tmp2 = 0;

	      for(int c=1; c <= nclasses; c++)
		{
		  tmp(c,c) = precision_tildew((c-1)*num_superthreshold+r,(c-1)*num_superthreshold+r);
		  tmp2(c,c) = D((c-1)*num_superthreshold+r,(c-1)*num_superthreshold+r);
		  for(int k=c+1; k <= nclasses; k++)
		    {
		      tmp(k,c) = precision_tildew((c-1)*num_superthreshold+r,(k-1)*num_superthreshold+r);
		    }
		}

	      trace_new += (tmp.i()*tmp2).Trace();
	  }
	//	OUT(trace_new);
      }
	
    trace_covariance_tildew_D = trace_new;	

    OUT(trace_covariance_tildew_D);  
  }

  ReturnMatrix Mixture_Model::transform(const RowVector& wtilde,float lambda, float log_bound)
  {
    // converts from wtilde (aka x) to w
    RowVector y = wtilde;
    RowVector ret_w = logistic_transform(y,lambda,log_bound);
    ret_w.Release();
    return ret_w;    
  }

  ReturnMatrix Mixture_Model::sum_transform(const RowVector& wtilde, float log_bound)
  {
    //Tracer_Plus trace("Mixture_Model::sum_transform");

    // converts from wtilde (aka x) to y
    RowVector ret_y  = log_bound*wtilde/std::sqrt(wtilde.SumSquare());    
    
    ret_y.Release();
    return ret_y;    
  }

  ReturnMatrix Mixture_Model::logistic_transform(const RowVector& y,float lambda,float log_bound)
  {
    //    Tracer_Plus trace("Mixture_Model::logistic_transform");

    // converts from y to w

    int nclasses = y.Ncols();
    float sum = 0.0;
    RowVector ret_weights(nclasses);
    ret_weights = 0;

    float phi = lambda*log_bound;
    for(int c=0; c < nclasses; c++)
      {
	sum += boundexp(y(c+1)/phi);
      }
    
    for(int c=0; c < nclasses; c++)
      {
	ret_weights(c+1) = boundexp(y(c+1)/phi)/sum;	
      }
    
    ret_weights.Release();
    return ret_weights;
    
  }

  ReturnMatrix Mixture_Model::inv_transform(const RowVector& w,float lambda,float log_bound)
  {
    //Tracer_Plus trace("Mixture_Model::inv_transform");

    // converts from w to wtilde (aka x)

    int nclasses = w.Ncols();
    
    RowVector ret_wtilde(nclasses);
    ret_wtilde = 0;

    int max = 1;
    for(int c=0; c < nclasses; c++)
      {
	if(w(c+1)>w(max)) max = c+1;
      }
    ret_wtilde(max) = 1;
    
    ret_wtilde.Release();
    return ret_wtilde;
    
  }  

  void Mixture_Model::save(volumeinfo& volinfo) 
  {
    Tracer_Plus trace("Mixture_Model::save");
    
    update_theta();

    save_volume(spatial_data, LogSingleton::getInstance().appendDir("spatial_data"),volinfo);
    save_volume(mask, LogSingleton::getInstance().appendDir("mask"),volinfo);

    // save distribution params
    ColumnVector means(nclasses);
    ColumnVector vars(nclasses);
    ColumnVector props(nclasses);
    means = 0;
    vars = 0;
    props = 0;

    for(int c = 0; c < nclasses; c++)
      {
	means(c+1) = dists[c]->getmean();
	vars(c+1) = dists[c]->getvar();
	props(c+1) = dists[c]->getprop();
      }

    for(int c=0; c < nclasses; c++)
      {
	write_ascii_matrix(means,LogSingleton::getInstance().appendDir("mu_mean"));
	write_ascii_matrix(vars,LogSingleton::getInstance().appendDir("var_mean"));
	write_ascii_matrix(props,LogSingleton::getInstance().appendDir("prop_mean"));
      }

    // save weights
    //    vector<volume<float> > w_means(nclasses);   
    vector<ColumnVector> weights;
    
    get_weights(weights);

//     OUT("Calculating precision matrix");
//     calculate_tildew_precision();
//     OUT("Calculating weights");
//     get_weights2(weights,30);
    
    for(int c=0; c < nclasses; c++)
      {
	w_means[c].reinitialize(xsize,ysize,zsize);
	w_means[c] = 0.0;
	
	for(int z = 0; z < mask.zsize(); z++)    
	  for(int y = 0; y < mask.ysize(); y++)	
	    for(int x = 0; x < mask.xsize(); x++)
	      if(mask(x,y,z))
		{
		  w_means[c](x,y,z) =  weights[c](indices(x,y,z));		    		  
		}

	copybasicproperties(spatial_data,w_means[c]);		
	save_volume(w_means[c], LogSingleton::getInstance().appendDir("w"+num2str(c+1)+"_mean"),volinfo);	
      }           

    // convergence stuff:
    write_vector(meanhist, LogSingleton::getInstance().appendDir("meanhist"));

    if(!nonspatial)
      {
	miscplot newplot;
	newplot.add_xlabel("Iterations");    
	newplot.set_xysize(610,300);
	newplot.timeseries(vector2ColumnVector(mrf_precision_hist).t(), LogSingleton::getInstance().appendDir("mrfprechist"), "MRF Precision", 0,400,3,0,false);
      }

    write_vector(mrf_precision_hist, LogSingleton::getInstance().appendDir("mrf_precision_hist"));

  }

  void Mixture_Model::get_weights(vector<ColumnVector>& weights)
  {
    weights.resize(nclasses);
    for(int c = 0; c < nclasses; c++)
      {
	weights[c].ReSize(num_superthreshold);
	weights[c] = 0;
      } 

    for(int r = 1; r<=num_superthreshold; r++)
      {       
	RowVector wtilde(nclasses);
	wtilde = 0;
	
	for(int c=0; c < nclasses; c++)
	  wtilde(c+1) = m_tildew(c*num_superthreshold+r);

	RowVector w = Mixture_Model::transform(wtilde,lambda,log_bound);

//  	OUT(r);
//  	OUT(wtilde);
//  	OUT(w);

	for(int c=0; c < nclasses; c++)
	  weights[c](r) = w(c+1);
      }
  }

  void Mixture_Model::get_weights2(vector<ColumnVector>& weights, int nsamps)
  {

    weights.resize(nclasses);
    for(int c = 0; c < nclasses; c++)
      {
	weights[c].ReSize(num_superthreshold);
	weights[c] = 0;
      } 

    for(int r = 1; r<=num_superthreshold; r++)
      {       
	RowVector wtilde(nclasses);	
	wtilde = 0;
	SymmetricMatrix wtildecov(nclasses);
	wtildecov = 0;
	
	for(int c=0; c < nclasses; c++)
	  {
	    wtilde(c+1) = m_tildew(c*num_superthreshold+r);
	    wtildecov(c+1,c+1) = 1.0/precision_tildew(c*num_superthreshold+r,c*num_superthreshold+r);	    
	  }
	
	Matrix wtildesamps = mvnrnd(wtilde,wtildecov,nsamps);
	
	RowVector w(nclasses);
	w = 0;
	for(int s=1; s <= nsamps; s++)
	  {
	     w += Mixture_Model::transform(wtildesamps.Row(s),lambda,log_bound);
	  }
	
	for(int c=0; c < nclasses; c++)
	  weights[c](r) = w(c+1)/float(nsamps);
      }
  }

  void ggmfit(const RowVector& dat, vector<Distribution*>& pdists)
  {// fit a mixture of a Gaussian and multiple Gamma functions to the histogram
  
    //normalise data 
    float datamean = mean(dat,2).AsScalar();
    float datastdev= stdev(dat,2).AsScalar();
    RowVector data=(dat - datamean)/datastdev;

    int numdata = dat.Ncols();
    int nummix = 3;
    RowVector means(nummix);
    RowVector vars(nummix);
    RowVector props(nummix);
    means = 0;
    vars = 0;
    props = 0;
//      Params=zeros(1,nummix);
    //    float logprobY = 1.0;

    props = std::pow(float(nummix),float(-1.0));

    Matrix tmp1;
    tmp1 = data * data.t() / numdata;
    vars = tmp1.AsScalar();

    float Dmin1, Dmax1, IntSize;
    Dmin1 =  min(data).AsScalar(); Dmax1 = max(data).AsScalar();
    IntSize = Dmax1 / nummix;

    means(1)=mean(data,2).AsScalar(); 
    for (int ctr=2; ctr <= means.Ncols(); ctr++){
      means(ctr) =  Dmax1 - (ctr - 1.5) * IntSize; 
    }
    means(2)=means(1)+2*sqrt(vars(1));
    //means(2)=means(1)+ 0.6*(Dmax-means(1));
    if(nummix>2)
      //means(3)=means(1)-0.6*(means(1)-Dmin);
            means(3)=means(1)-2*sqrt(vars(1));

//      epsilon = eps;
//      if(epsilon >=0 && epsilon < 0.0000001)
//        {epsilon = std::log(float(dat.Ncols()))/1000 ;}


    // ggmfit
    float log_p_y_theta = 1.0;
    float old_ll = 2.0;
    float g_eps = 0.000001;
    int it_ctr = 0;
    double Dmax, Dmin;
   
    Dmax = 2 * data.Maximum();
    Dmin = -2 * data.Minimum();

    //resize means, vars and props
    if(nummix > 3)
      nummix = 3;
    means = means.Columns(1,nummix);
    vars  = vars.Columns(1,nummix);
    props = props.Columns(1,nummix);

    means(1) = -2*mean(data,2).AsScalar();

    Matrix prob_K__y_theta;
    Matrix kdata;
    RowVector prob_Y__theta;RowVector Nbar;
    RowVector mubar;RowVector sigmabar;RowVector pibar;
    Matrix p_ygx(nummix,numdata);
    //    float offset = 0.0;
    float const2;
    Matrix negdata(data);
    negdata = -data;

    while((it_ctr<10) ||
	  ((std::abs(old_ll - log_p_y_theta)>g_eps) && (it_ctr<100)))
      { // fit GGM
	
 	it_ctr++;
	//offset = (std::min(0.2,1-props(1)))*std::sqrt(vars(1));

// 	//make sure all variances are acceptable
 	for(int ctr1=1; ctr1 <=nummix; ctr1++)
 	  if(vars(ctr1)<0.0001){
 	    vars(ctr1) = 0.0001;
 	  }

 	p_ygx = 0.0;
 	p_ygx.Row(1) << normpdf(data,means(1),vars(1));
       
 	const2 = (2.6-props(1))*sqrt(vars(1))+means(1); //min. nht level
 
	means(2) = (std::max(means(2), std::max(0.001,
	   0.5 * ( const2 + std::sqrt( const2*const2 + 4*vars(2))))));
	vars(2)  = std::max(std::min(vars(2), 0.5*std::pow(means(2),2)),0.0001);
	p_ygx.Row(2) << gammapdf(data,means(2),vars(2));
   
 	if(nummix>2){
	  const2 = (2.6-props(1))*sqrt(vars(1))-means(1);
	
	  means(3) = -(std::max(-means(3), std::max(0.001,
	      0.5 * ( const2 + std::sqrt( const2*const2 + 4*vars(3))))));
	  vars(3)  = std::max(std::min(vars(3), 0.5*std::pow(means(3),2)),0.0001);
 	  p_ygx.Row(3) << gammapdf(negdata,-means(3),vars(3));
	}

 	tmp1 = SP(props.t()*ones(1,numdata),p_ygx);
 	prob_Y__theta << sum(tmp1,1);
	
	//deal with non-modelled voxels
	for(int ctr=1; ctr<=tmp1.Ncols(); ctr++)
	  if(prob_Y__theta(ctr) < 0.0001)
	    prob_Y__theta(ctr) = 0.0001;

 	old_ll = log_p_y_theta;
 	log_p_y_theta = log(prob_Y__theta).Sum();
	// 	cerr << "calculated log_prob_Y__theta" <<endl;
	// 	cerr << old_ll << "   " << log_p_y_theta << "   " 
	//	cerr << float(std::abs(old_ll - log_p_y_theta)) << endl;
 	if((it_ctr<10) ||
	   ((std::abs(old_ll - log_p_y_theta)>g_eps) && (it_ctr<100))){//update
	  
 	  prob_K__y_theta = SP(tmp1,pow(ones(nummix,1)*prob_Y__theta,-1));
 	  Nbar << sum(prob_K__y_theta,2).t();
	  for(int ctr=1; ctr<=Nbar.Ncols(); ctr++)
	    if(Nbar(ctr) < 0.0001 * numdata)
	      Nbar = Nbar + 0.0001;
 	  pibar= Nbar / numdata;
	  // 	  cerr << "pibar :" << pibar << endl;
 	  kdata = ones(nummix,1)*data;
 	  mubar <<SP(sum(SP(kdata,prob_K__y_theta),2).t(),pow(Nbar,-1)); 
	  // 	  cerr << "mubar :" << mubar << endl;

 	  kdata -= mubar.t()*ones(1,numdata);
 	  kdata = pow(kdata,2);
 	  sigmabar << SP(sum(SP(kdata,prob_K__y_theta),2).t(),pow(Nbar,-1));
      
 	  means = mubar;
 	  vars  = sigmabar;
 	  props = pibar;
 	}//update
      } //while loop


    data = data*datastdev+datamean;
    means = means*datastdev+datamean;

    for(int c=0; c < nummix; c++)
      vars(c+1) = Sqr(std::sqrt(vars(c+1))*datastdev);

    if(((it_ctr%1)==0)||(it_ctr==1)){
      string pad = "-";
      miscplot newplot;

      newplot.ggmfit(data,means,vars,props,
		     LogSingleton::getInstance().appendDir("init_mmfit.png"),string("Initial Fit"),0.0);
    }    

    for(unsigned int c=0; c < pdists.size(); c++)
      {	
	pdists[c]->setparams(means(c+1),vars(c+1),props(c+1));
	  
 	OUT(c);
 	OUT(pdists[c]->getmean());
 	OUT(pdists[c]->getvar());
      }
  }

  void plot_ggm(const vector<volume<float> >& w_means, const vector<Distribution*>& dists, const volume<int>& mask, const ColumnVector& Y)
  {
    OUT("plot_ggm");
    // update means and vars using ML estimates, given weights are known    
    int nclasses = w_means.size();

    OUT(nclasses);

    for(int c = 0; c < nclasses; c++)
      {	
	OUT(c);
	float sumweights = 0;
	int num_superthreshold = 0;

	for(int z = 0; z < mask.zsize(); z++)    
	  for(int y = 0; y < mask.ysize(); y++)	
	    for(int x = 0; x < mask.xsize(); x++)
	      if(mask(x,y,z))
		{
		  float wtmp = w_means[c](x,y,z);
		  sumweights += wtmp;
		  num_superthreshold++;
		}

	OUT(num_superthreshold);

	dists[c]->setparams(dists[c]->getmean(),dists[c]->getvar(),sumweights/num_superthreshold);
      }

    int nummix = 3;
    RowVector means(nummix);
    RowVector vars(nummix);
    RowVector props(nummix);
    means = 0;
    vars = 0;
    props = 0;

    for(int c = 0; c < nclasses; c++)
      {
	means(c+1) = dists[c]->getmean();
	vars(c+1) = dists[c]->getvar();
	props(c+1) = dists[c]->getprop();
      }

    if(nclasses==2)
      {
	means(3) = 0;
	vars(3) = 0.1;
	props(3) = 0;
      }

    OUT(means);
    OUT(vars);
    OUT(props);

    miscplot newplot;
 
    newplot.ggmfit(Y.t(),means,vars,props,LogSingleton::getInstance().appendDir("final_mmfit.png"),string("Final Fit"),0.0);
    
  }

  void make_ggmreport(const vector<volume<float> >& w_means, const vector<Distribution*>& dists, const volume<int>& mask, const volume<float>& spatial_data, bool zfstatmode, volumeinfo& volinfo, bool overlay, const volume<float>& epivol, float thresh, bool nonspatial, const string& data_name)
  {
    int nclasses = dists.size();

    RowVector means(nclasses);
    RowVector vars(nclasses);
    RowVector props(nclasses);
    means = 0;
    vars = 0;
    props = 0;

    for(int c = 0; c < nclasses; c++)
      {
	means(c+1) = dists[c]->getmean();
	vars(c+1) = dists[c]->getvar();
	props(c+1) = dists[c]->getprop();
      }

    LogSingleton::getInstance().setLogFile(string("MM.html"));

    Log& htmllog = LogSingleton::getInstance();
      {	
//  	OUT(htmllog.getDir());
//  	OUT(htmllog.getLogFileName());

	htmllog << "<HTML> " << endl
		<< "<TITLE>Mixture Model fit for" << data_name << "</TITLE>" << endl
		<< "<BODY BACKGROUND=\"file:" << getenv("FSLDIR") 
		<< "/doc/images/fsl-bg.jpg\">" << endl 
		<< "<hr><CENTER><H1>Mixture Model fit for<br>" << data_name << " </H1>"<< endl;
     	
	htmllog << "<hr><p>" << endl;
      }

      {
//        volume<float> map1;
//        volume<float> map2;

//        map1 = threshold(spatial_data,float(0.0), 
//  		       spatial_data.max());
//        map2 = threshold(spatial_data,spatial_data.min(), 
//  		       float(-0.0));
      
        volume<float> newvol; 
        miscpic newpic;
      
	newvol = spatial_data;

 	char instr[10000];
	
 	sprintf(instr," ");
 	strcat(instr,"-s 2 ");	
	strcat(instr,(string("-i ")+num2str(spatial_data.min())+" "+num2str(spatial_data.max())+" ").c_str());
 	strcat(instr,"-A 950 ");
 	strcat(instr,LogSingleton::getInstance().appendDir("spatial_data.png").c_str());

 	newpic.set_title(string("Raw spatial map"));
	
 	newpic.slicer(newvol, instr, &volinfo);
      

	htmllog << "<img BORDER=0 SRC=\"spatial_data.png\"><p>" << endl;
      }

      if(overlay)
	{	
	  volume<float> map;	  

	  map = w_means[1];
	
 	  if(thresh>0)
 	    map.threshold(0.5);	 
	  else
	    thresh=0;

	  volume<float> newvol; 
	  miscpic newpic;

	  volume<float> epivoltmp = epivol;

	  save_volume(map, LogSingleton::getInstance().appendDir("map"),volinfo);
	  save_volume(epivoltmp, LogSingleton::getInstance().appendDir("epivol"),volinfo);
	 
	  newpic.overlay(newvol, epivoltmp, map, map, 
			 epivol.percentile(0.01),
			 epivol.percentile(0.99),
			 float(thresh), float(1.0), float(0.0), float(0.0),
			 0, 0, &volinfo);
		
	  char instr[10000];
	
	  sprintf(instr," ");
	  strcat(instr,"-l render1 -s 2 ");
	  strcat(instr,"-A 950 ");
	  strcat(instr,LogSingleton::getInstance().appendDir("actprobmap.png").c_str());     

	  string tit = "Activation prob map";
	  if(thresh>0)
	    tit += " thresholded at p>" + num2str(thresh);
	  newpic.set_title(tit);	    
	
	  newpic.set_cbar(string("y"));
	  newpic.slicer(newvol, instr, &volinfo); 
	
	  htmllog << "<img BORDER=0 SRC=\"actprobmap.png\">" << endl;
	  htmllog << "<p>" << endl;

	}
      else
	{			
	  volume<float> newvol; 
	  miscpic newpic;

	  newvol = w_means[1];

	  if(thresh>0)
	    newvol.threshold(0.5);

	  char instr[10000];
	
	  	      //volume<float> tmp = newvol/newvol.max();
	      volume<float> tmp = spatial_data;
// 	      newpic.overlay(newvol, newvol, tmp, tmp,
// 			     float(0),float(1),
// 			     float(2), float(3), float(0.0), float(0.0),
// 			     0, 0, &volinfo);
	      newpic.overlay(newvol, tmp,newvol, newvol,
			     spatial_data.percentile(0.01),
			     spatial_data.percentile(0.99),
			     float(thresh), float(1.0), float(0.0), float(0.0),
			     0, 0, &volinfo);

	  sprintf(instr," ");
	  strcat(instr,"-s 2 ");
	  //	strcat(instr,"-i 0 1 ");
	  strcat(instr,"-A 950 ");
	  strcat(instr,LogSingleton::getInstance().appendDir("actprobmap.png").c_str());      

	  string tit = "Activation prob map";
	  if(thresh>0)
	    tit += " thresholded at p>" + num2str(thresh);
	  newpic.set_title(tit);	    
	  
	  newpic.set_cbar(string("y"));
	  newpic.slicer(newvol, instr, &volinfo); 
		
	  htmllog << "<img BORDER=0 SRC=\"actprobmap.png\">" << endl;
	  htmllog << "<p>" << endl;
	}
      
      if(!zfstatmode)
	{	
	  // Output deactivation
	  if(overlay)
	    {	
	      volume<float> map;
	      map = w_means[2];
	
	      if(thresh>0)
		map.threshold(0.5);
	      else
		thresh=0;

	      volume<float> newvol; 
	      
	      miscpic newpic;

	      volume<float> epivoltmp = epivol;

	      newpic.overlay(newvol, epivoltmp, map, map, 
			     epivol.percentile(0.01),
			     epivol.percentile(0.99),
			     float(thresh), float(1.0), float(0.0), float(0.0),
			     0, 0, &volinfo);
		
	      char instr[10000];
	
	      sprintf(instr," ");
	      strcat(instr,"-s 2 ");
	      strcat(instr,"-A 950 ");
	      strcat(instr,LogSingleton::getInstance().appendDir("deactprobmap.png").c_str());

	      string tit = "Deactivation prob map";
	      if(thresh>0)
		tit += " thresholded at p>" + num2str(thresh);
	      newpic.set_title(tit);
	
	      newpic.set_cbar(string("y"));
	      newpic.slicer(newvol, instr, &volinfo); 
	
	      htmllog << "<img BORDER=0 SRC=\"deactprobmap.png\">" << endl;
	      htmllog << "<p>" << endl;

	    }
	  else
	    {	      
	    	    
	      volume<float> newvol; 
	      miscpic newpic;
	
	      newvol = w_means[2];
	      if(thresh>0)
		newvol.threshold(0.5);

	      char instr[10000];
	
	      //volume<float> tmp = newvol/newvol.max();
	      volume<float> tmp = spatial_data;
// 	      newpic.overlay(newvol, newvol, tmp, tmp,
// 			     float(0),float(1),
// 			     float(2), float(3), float(0.0), float(0.0),
// 			     0, 0, &volinfo);
	      newpic.overlay(newvol, tmp,newvol, newvol,
			     spatial_data.percentile(0.01),
			     spatial_data.percentile(0.99),
			     float(thresh), float(1.0), float(0.0), float(0.0),
			     0, 0, &volinfo);

	      sprintf(instr," ");
	      strcat(instr,"-s 2 ");
	      //	      strcat(instr,"-i 0 1 ");
	      strcat(instr,"-A 950 ");
	      strcat(instr,LogSingleton::getInstance().appendDir("deactprobmap.png").c_str());      
	      string tit = "Deactivation prob map";
	      if(thresh>0)
		tit += " thresholded at p>" + num2str(thresh);
	      newpic.set_title(tit);

	      newpic.set_cbar(string("y"));
	      newpic.slicer(newvol, instr, &volinfo); 
	      htmllog << "<img BORDER=0 SRC=\"deactprobmap.png\">" << endl;
	      htmllog << "<p>" << endl;
	    }
	}
      
      {//Output GGM fit
	miscplot newplot;

	htmllog << "<A><img BORDER=0 SRC=\"final_mmfit.png\"></A><p>" << endl;
      }
      
      {
	htmllog << "<br> Mixture Model fit <br>" << endl
		<< "<br> &nbsp; Means :  " << means << endl
		<< "<br> &nbsp;  Vars  :  " << vars  << endl
		<< "<br> &nbsp;  Prop. :  " << props    << endl;	
      }

      if(!nonspatial)
	{
	  // output MRF precision convergence plot
	  htmllog << "<p><A><img BORDER=0 SRC=\"mrfprechist.png\"></A><p>" << endl;
	  htmllog << "</CENTER> Spatial mixture modelling based on: <br> Mixture Models with Adaptive Spatial Regularisation for Segmentation with an Application to FMRI Data; Woolrich, M., Behrens, T., Beckmann, C., and Smith, S.; IEEE TMI In Press 2004.";

	}
      else
	{
	  htmllog << "</CENTER> Non-spatial mixture modelling based on: <br> Mixture Models with Adaptive Spatial Regularisation for Segmentation with an Application to FMRI Data; Woolrich, M., Behrens, T., Beckmann, C., and Smith, S.; IEEE TMI In Press 2004.";
	
	}

      { 
	htmllog<< "<HR><FONT SIZE=1>This page produced automatically by "
	       << "mm </A>" 
	       << " - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL - "
	       << "FMRIB Software Library</A>.</FONT>" << endl
	       << "</BODY></HTML>" << endl;
      } 
  }
  

}
