#

#   iso - this changes the voxel size to make the image isotropic
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   http://www.fmrib.ox.ac.uk/fsl
#   fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   
#   LICENCE
#   
#   FMRIB Software Library, Release 3.2beta (c) 2004, The University of
#   Oxford (the "Software")
#   
#   The Software remains the property of the University of Oxford ("the
#   University").
#   
#   The Software is distributed "AS IS" under this Licence solely for
#   non-commercial use in the hope that it will be useful, but in order
#   that the University as a charitable foundation protects its assets for
#   the benefit of its educational and research purposes, the University
#   makes clear that no condition is made or to be implied, nor is any
#   warranty given or to be implied, as to the accuracy of the Software,
#   or that it will be suitable for any particular purpose or for use
#   under any specific conditions. Furthermore, the University disclaims
#   all responsibility for the use which is made of the Software. It
#   further disclaims any liability for the outcomes arising from using
#   the Software.
#   
#   The Licensee agrees to indemnify the University and hold the
#   University harmless from and against any and all claims, damages and
#   liabilities asserted by third parties (including claims for
#   negligence) which arise directly or indirectly from the use of the
#   Software or the sale of any products based on the Software.
#   
#   No part of the Software may be reproduced, modified, transmitted or
#   transferred in any form or by any means, electronic or mechanical,
#   without the express permission of the University. The permission of
#   the University is not required if the said reproduction, modification,
#   transmission or transference is done without financial return, the
#   conditions of this Licence are imposed upon the receiver of the
#   product, and all original and amended source code is included in any
#   transmitted product. You may be held legally responsible for any
#   copyright infringement that is caused or encouraged by your failure to
#   abide by these terms and conditions.
#   
#   You are not permitted under this Licence to use this Software
#   commercially. Use for which any financial return is received shall be
#   defined as commercial use, and includes (1) integration of all or part
#   of the source code or the Software into a product for sale or license
#   by or on behalf of Licensee to third parties or (2) use of the
#   Software or any derivative of it for research with the final aim of
#   developing software products for sale or license to a third party or
#   (3) use of the Software or any derivative of it for research with the
#   final aim of developing non-software products for sale or license to a
#   third party, or (4) use of the Software to provide any service to an
#   external organisation for which payment is received. If you are
#   interested in using the Software commercially, please contact Isis
#   Innovation Limited ("Isis"), the technology transfer company of the
#   University, to negotiate a licence. Contact details are:
#   innovation@isis.ox.ac.uk quoting reference DE/1112.

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

proc makeiso { Folder Page zadd } {

    #{{{ find image properties

    set depth 1
    MxGetDimensions $Page width height depth

    puts "Dimensions=($width,$height,$depth)"

    set zscale 1
    MxGetScale $Page xscale yscale zscale

    puts "Voxel=($xscale,$yscale,$zscale)"

#}}}
    #{{{ setup scaling

    if { $xscale < $yscale } {
	set cubel $xscale
    } else {
	set cubel $yscale
    }	
    if { $zscale < $cubel } {
	set cubel $zscale
    }

    puts "Cubel side length=$cubel"

    set xscaling [ expr double($xscale) / $cubel ]
    set yscaling [ expr double($yscale) / $cubel ]
    set zscaling [ expr double($zscale) / $cubel ]

    puts "Scaling=($xscaling,$yscaling,$zscaling)"

#}}}
    #{{{ COMMENT setup new dimensions OLD

#    set newwidth  [ expr int( $width  * $xscaling ) ]
#    set newheight [ expr int( $height * $yscaling ) ]
#    set newdepth  [ expr int( $depth  * $zscaling ) ]
#
#    puts "New Dimensions=($newwidth,$newheight,$newdepth)"
#    
#    if { $zadd > -0.001 } {
#	set zadd [ expr int( double($zadd) / $cubel ) ]
#	puts "Adding $zadd slices to top and bottom:"
#
#	set newdepth [ expr $newdepth + ( $zadd * 2 ) ]
#    } else {
#	set newdepth [ expr 0 - $zadd ]
#    }
#
#    puts "New Dimensions=($newwidth,$newheight,$newdepth)"
#

#}}}
    #{{{ setup new dimensions

    set newwidth  [ expr int( $width  * $xscaling ) ]
    set newheight [ expr int( $height * $yscaling ) ]
    set newdepth  [ expr int( $depth  * $zscaling ) ]

    puts "New Dimensions=($newwidth,$newheight,$newdepth)"
    
# this line took in zadd as mm rather than voxls - commented it out
#    set zadd [ expr int( double($zadd) / $cubel ) ]
    puts "Adding $zadd slices to top and bottom:"

    set newdepth [ expr $newdepth + ( $zadd * 2 ) ]

    puts "New Dimensions=($newwidth,$newheight,$newdepth)"

#}}}

    MxInterpolationMode BTL

    MxReslice $Page "{Cx [ expr ( $width - 1.0 ) / 2.0 ] } {Cy [ expr ( $height - 1.0 ) / 2.0 ] } \
	    {Cz [ expr ( $depth - 1.0 ) / 2.0 ] } {Rx 0} {Ry 0} {Rz 0} {Sx $xscaling} {Sy $yscaling} {Sz $zscaling} \
	    {Tx 0} {Ty 0} {Tz 0} {Dx $newwidth} {Dy $newheight} {Dz $newdepth } {RotMethod M}" IsoPage

    return $IsoPage
}
